"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isMessage = isMessage;
exports.isCommandMessage = isCommandMessage;
exports.isCreatePeerConnectionMessage = isCreatePeerConnectionMessage;
exports.isSignalingMessage = isSignalingMessage;
exports.isClosePeerConnectionMessage = isClosePeerConnectionMessage;
exports.isConfigurationMessage = isConfigurationMessage;
exports.isExperimentStatusChangedMessage = isExperimentStatusChangedMessage;
exports.isWebSocketMessage = isWebSocketMessage;
const zod_1 = require("zod");
// define message schema
const messageSchema = zod_1.z.object({
    messageType: zod_1.z.string(),
});
function isMessage(input) {
    return messageSchema.safeParse(input).success;
}
// define command message schema
const commandMessageSchema = messageSchema.extend({
    messageType: zod_1.z.literal('command'),
    command: zod_1.z.string(),
});
function isCommandMessage(input) {
    return commandMessageSchema.safeParse(input).success;
}
// define service configuration schema
const serviceConfigurationSchema = zod_1.z.object({
    serviceType: zod_1.z.string(),
    serviceId: zod_1.z.string(),
    remoteServiceId: zod_1.z.string(),
});
// define create peerconnection message schemas
const createPeerConnectionMessageBaseSchema = commandMessageSchema.extend({
    command: zod_1.z.literal('createPeerconnection'),
    connectionType: zod_1.z.string(),
    connectionUrl: zod_1.z.string(),
    services: zod_1.z.array(serviceConfigurationSchema),
    tiebreaker: zod_1.z.boolean(),
});
const createWebSocketPeerConnectionMessageSchema = createPeerConnectionMessageBaseSchema.extend({
    connectionType: zod_1.z.literal('websocket'),
    connectionOptions: zod_1.z.object({
        webSocketUrl: zod_1.z.string(),
    }),
});
const createWebRTCPeerConnectionMessageSchema = createPeerConnectionMessageBaseSchema.extend({
    connectionType: zod_1.z.literal('webrtc'),
    connectionOptions: zod_1.z.object({
        iceServers: zod_1.z.optional(zod_1.z.array(zod_1.z.object({
            urls: zod_1.z.union([zod_1.z.string(), zod_1.z.array(zod_1.z.string())]),
            username: zod_1.z.optional(zod_1.z.string()),
            credential: zod_1.z.optional(zod_1.z.string()),
        }))),
    }),
});
const createLocalPeerConnectionMessageSchema = createPeerConnectionMessageBaseSchema.extend({
    connectionType: zod_1.z.literal('local'),
});
const createPeerConnectionMessageSchema = zod_1.z.union([
    createWebSocketPeerConnectionMessageSchema,
    createWebRTCPeerConnectionMessageSchema,
    createLocalPeerConnectionMessageSchema,
]);
function isCreatePeerConnectionMessage(input) {
    return createPeerConnectionMessageSchema.safeParse(input).success;
}
// define signaling message schema
const signalingMessageSchema = messageSchema.extend({
    messageType: zod_1.z.literal('signaling'),
    connectionUrl: zod_1.z.string(),
    signalingType: zod_1.z.union([
        zod_1.z.literal('options'),
        zod_1.z.literal('offer'),
        zod_1.z.literal('answer'),
        zod_1.z.literal('candidate'),
    ]),
    content: zod_1.z.unknown(),
});
function isSignalingMessage(input) {
    return signalingMessageSchema.safeParse(input).success;
}
// define close peerconnection message schema
const closePeerConnectionMessageSchema = commandMessageSchema.extend({
    command: zod_1.z.literal('closePeerconnection'),
    connectionUrl: zod_1.z.string(),
});
function isClosePeerConnectionMessage(input) {
    return closePeerConnectionMessageSchema.safeParse(input).success;
}
// define connection state changed message schema
const connectionStateChangedMessageSchema = messageSchema.extend({
    messageType: zod_1.z.literal('connection-state-changed'),
    connectionUrl: zod_1.z.string(),
    status: zod_1.z.union([
        zod_1.z.literal('new'),
        zod_1.z.literal('connecting'),
        zod_1.z.literal('connected'),
        zod_1.z.literal('disconnected'),
        zod_1.z.literal('failed'),
        zod_1.z.literal('closed'),
    ]),
});
// define configuration message schema
const configurationMessageSchema = messageSchema.extend({
    messageType: zod_1.z.literal('configuration'),
    configuration: zod_1.z.record(zod_1.z.string(), zod_1.z.unknown()),
});
function isConfigurationMessage(input) {
    return configurationMessageSchema.safeParse(input).success;
}
// define experiment status changed message schema
const experimentStatusChangedMessageSchema = messageSchema.extend({
    messageType: zod_1.z.literal('experiment-status-changed'),
    status: zod_1.z.union([
        zod_1.z.literal('created'),
        zod_1.z.literal('booked'),
        zod_1.z.literal('setup'),
        zod_1.z.literal('running'),
        zod_1.z.literal('finished'),
    ]),
    message: zod_1.z.optional(zod_1.z.string()),
});
function isExperimentStatusChangedMessage(input) {
    return experimentStatusChangedMessageSchema.safeParse(input).success;
}
// define service description schema
const serviceDescriptionSchema = zod_1.z.object({
    serviceType: zod_1.z.string(),
    serviceId: zod_1.z.string(),
    serviceDirection: zod_1.z.string(),
    interfaces: zod_1.z.array(zod_1.z.unknown()),
});
// define device description schema
const deviceDescriptionSchema = zod_1.z.object({
    services: zod_1.z.array(serviceDescriptionSchema),
});
// define websocket message type schema
const webSocketMessageSchema = zod_1.z.union([
    zod_1.z.object({
        type: zod_1.z.literal('string'),
        channel: zod_1.z.string(),
        content: zod_1.z.string(),
    }),
    zod_1.z.object({
        type: zod_1.z.union([zod_1.z.literal('arrayBuffer'), zod_1.z.literal('blob')]),
        channel: zod_1.z.string(),
        content: zod_1.z.array(zod_1.z.number()),
    }),
    zod_1.z.object({
        type: zod_1.z.literal('arrayBufferView'),
        channel: zod_1.z.string(),
        content: zod_1.z.object({
            buffer: zod_1.z.array(zod_1.z.number()),
            byteLength: zod_1.z.number(),
            byteOffset: zod_1.z.number(),
        }),
    }),
]);
function isWebSocketMessage(input) {
    return webSocketMessageSchema.safeParse(input).success;
}
//# sourceMappingURL=deviceMessages.js.map